@@ -143,8 +143,7 @@ impl AudioBuffer {
     }
 
     /// Split an AudioBuffer in two at the given index.
-    pub fn split_off(&mut self, index: u32) -> AudioBuffer {
-        let index = index as usize;
+    pub fn split_off(&mut self, index: usize) -> AudioBuffer {
         let sample_rate = self.sample_rate();
 
         let channels: Vec<_> = self
@@ -408,7 +407,7 @@ pub struct Resampler<I> {
     /// desired sample rate
     sample_rate: SampleRate,
     /// desired sample length
-    sample_len: u32,
+    sample_len: usize,
     /// input stream
     input: I,
     /// internal buffer
@@ -416,7 +415,7 @@ pub struct Resampler<I> {
 }
 
 impl<M: MediaStream> Resampler<M> {
-    pub fn new(sample_rate: SampleRate, sample_len: u32, input: M) -> Self {
+    pub fn new(sample_rate: SampleRate, sample_len: usize, input: M) -> Self {
         Self {
             sample_rate,
             sample_len,
@@ -442,13 +441,13 @@ impl<M: MediaStream> Iterator for Resampler<M> {
             Some(data) => data,
         };
 
-        while (buffer.sample_len() as u32) < self.sample_len {
+        while buffer.sample_len() < self.sample_len {
             // buffer is smaller than desired len
             match self.input.next() {
                 None => {
                     let padding = AudioBuffer::new(
                         buffer.number_of_channels(),
-                        self.sample_len as usize - buffer.sample_len(),
+                        self.sample_len - buffer.sample_len(),
                         self.sample_rate,
                     );
                     buffer.extend(&padding);
@@ -463,7 +462,7 @@ impl<M: MediaStream> Iterator for Resampler<M> {
             }
         }
 
-        if buffer.sample_len() as u32 == self.sample_len {
+        if buffer.sample_len() == self.sample_len {
             return Some(Ok(buffer));
         }
 
@@ -486,8 +485,8 @@ mod tests {
         assert_eq!(b.sample_len(), 10);
         assert_eq!(b.number_of_channels(), 2);
         assert_eq!(b.sample_rate().0, 44_100);
-        assert_float_eq!(b.channel_data(0).as_slice(), &[0.; 10][..], ulps_all <= 0);
-        assert_float_eq!(b.channel_data(1).as_slice(), &[0.; 10][..], ulps_all <= 0);
+        assert_float_eq!(b.channel_data(0).as_slice(), &[0.; 10][..], abs_all <= 0.);
+        assert_float_eq!(b.channel_data(1).as_slice(), &[0.; 10][..], abs_all <= 0.);
         assert_eq!(b.channels().get(2), None);
     }
 
@@ -512,19 +511,19 @@ mod tests {
         assert_float_eq!(
             b1.channel_data(0).as_slice(),
             &[0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 1., 1., 1., 1., 1.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
 
         let split = b1.split(8);
         assert_float_eq!(
             split[0].channel_data(0).as_slice(),
             &[0., 0., 0., 0., 0., 0., 0., 0.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
         assert_float_eq!(
             split[1].channel_data(0).as_slice(),
             &[0., 0., 1., 1., 1., 1., 1.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
     }
 
@@ -536,7 +535,7 @@ mod tests {
         assert_float_eq!(
             buffer.channel_data(0).as_slice(),
             &[1., 1., 2., 2., 3., 3., 4., 4., 5., 5.,][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
         assert_eq!(buffer.sample_rate().0, 200);
     }
@@ -549,7 +548,7 @@ mod tests {
         assert_float_eq!(
             buffer.channel_data(0).as_slice(),
             &[2., 4.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
         assert_eq!(buffer.sample_rate().0, 100);
     }
@@ -566,7 +565,7 @@ mod tests {
         assert_float_eq!(
             next.channel_data(0).as_slice(),
             &[1., 2., 3., 4., 5., 1., 2., 3., 4., 5.,][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
 
         let next = resampler.next().unwrap().unwrap();
@@ -574,7 +573,7 @@ mod tests {
         assert_float_eq!(
             next.channel_data(0).as_slice(),
             &[1., 2., 3., 4., 5., 0., 0., 0., 0., 0.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
 
         assert!(resampler.next().is_none());
@@ -595,7 +594,7 @@ mod tests {
         assert_float_eq!(
             next.channel_data(0).as_slice(),
             &[1., 2., 3., 4., 5.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
 
         let next = resampler.next().unwrap().unwrap();
@@ -603,7 +602,7 @@ mod tests {
         assert_float_eq!(
             next.channel_data(0).as_slice(),
             &[6., 7., 8., 9., 10.][..],
-            ulps_all <= 0
+            abs_all <= 0.
         );
 
         assert!(resampler.next().is_none());
