@@ -1,20 +1,34 @@
-//! The BaseAudioContext interface and the AudioContext and OfflineAudioContext types
+//! The `BaseAudioContext` interface and the `AudioContext` and `OfflineAudioContext` types
+#![warn(
+    clippy::all,
+    clippy::pedantic,
+    clippy::nursery,
+    clippy::perf,
+    clippy::missing_docs_in_private_items
+)]
 
 use std::ops::Range;
 use std::sync::atomic::{AtomicU64, Ordering};
 use std::sync::Arc;
 
 // magic node values
+/// Destination node id is always at index 0
 const DESTINATION_NODE_ID: u64 = 0;
+/// listener node id is always at index 1
 const LISTENER_NODE_ID: u64 = 1;
+/// listener audio parameters ids are always at index 2 through 12
 const LISTENER_PARAM_IDS: Range<u64> = 2..12;
 
 use crate::buffer::{AudioBuffer, ChannelConfigOptions, ChannelCountMode, ChannelInterpretation};
 use crate::graph::{NodeIndex, RenderThread};
 use crate::media::{MediaElement, MediaStream};
 use crate::message::ControlMessage;
-use crate::node::{self, AudioNode, PeriodicWave, PeriodicWaveOptions};
-use crate::param::{AudioParam, AudioParamOptions, AutomationEvent};
+use crate::node::{
+    self, AnalyserOptions, AudioBufferSourceNodeOptions, AudioNode, ChannelMergerOptions,
+    ChannelSplitterOptions, ConstantSourceOptions, DelayOptions, GainOptions, IirFilterOptions,
+    PannerOptions, PeriodicWave, PeriodicWaveOptions,
+};
+use crate::param::{AudioParam, AudioParamEvent, AudioParamOptions};
 use crate::process::AudioProcessor;
 use crate::spatial::{AudioListener, AudioListenerParams};
 use crate::{SampleRate, BUFFER_SIZE};
@@ -27,11 +41,14 @@ use cpal::{traits::StreamTrait, Stream};
 
 use crossbeam_channel::Sender;
 
-/// The BaseAudioContext interface represents an audio-processing graph built from audio modules
-/// linked together, each represented by an AudioNode. An audio context controls both the creation
+/// The `BaseAudioContext` interface represents an audio-processing graph built from audio modules
+/// linked together, each represented by an `AudioNode`. An audio context controls both the creation
 /// of the nodes it contains and the execution of the audio processing, or decoding.
+// the naming comes from the web audio specfication
+#[allow(clippy::module_name_repetitions)]
 #[derive(Clone)]
 pub struct BaseAudioContext {
+    /// inner makes `BaseAudioContextInner` cheap to clone
     inner: Arc<BaseAudioContextInner>,
 }
 
@@ -41,6 +58,7 @@ impl PartialEq for BaseAudioContext {
     }
 }
 
+/// Inner representation of the `BaseAudioContext`
 struct BaseAudioContextInner {
     /// sample rate in Hertz
     sample_rate: SampleRate,
@@ -56,59 +74,88 @@ struct BaseAudioContextInner {
     listener_params: Option<AudioListenerParams>,
 }
 
-/// Retrieve the BaseAudioContext from the concrete AudioContext
+/// Retrieve the `BaseAudioContext` from the concrete `AudioContext`
+#[allow(clippy::module_name_repetitions)]
 pub trait AsBaseAudioContext {
+    /// retrieves the `BaseAudioContext` associated with the concrete `AudioContext`
     fn base(&self) -> &BaseAudioContext;
 
-    /// Creates an OscillatorNode, a source representing a periodic waveform. It basically
+    /// Creates an `OscillatorNode`, a source representing a periodic waveform. It basically
     /// generates a tone.
     fn create_oscillator(&self) -> node::OscillatorNode {
-        node::OscillatorNode::new(self.base(), Default::default())
+        node::OscillatorNode::new(self.base(), None)
     }
 
-    /// Creates an GainNode, to control audio volume
+    /// Creates an `StereoPannerNode` to pan a stereo output
+    fn create_stereo_panner(&self) -> node::StereoPannerNode {
+        node::StereoPannerNode::new(self.base(), None)
+    }
+
+    /// Creates an `GainNode`, to control audio volume
     fn create_gain(&self) -> node::GainNode {
-        node::GainNode::new(self.base(), Default::default())
+        node::GainNode::new(self.base(), GainOptions::default())
     }
 
-    /// Creates an ConstantSourceNode, a source representing a constant value
+    /// Creates an `ConstantSourceNode`, a source representing a constant value
     fn create_constant_source(&self) -> node::ConstantSourceNode {
-        node::ConstantSourceNode::new(self.base(), Default::default())
+        node::ConstantSourceNode::new(self.base(), ConstantSourceOptions::default())
     }
 
-    /// Creates a DelayNode, delaying the audio signal
+    /// Creates an `IirFilterNode`
+    ///
+    /// # Arguments
+    ///
+    /// * `feedforward` - An array of the feedforward (numerator) coefficients for the transfer function of the IIR filter.
+    /// The maximum length of this array is 20
+    /// * `feedback` - An array of the feedback (denominator) coefficients for the transfer function of the IIR filter.
+    /// The maximum length of this array is 20
+    fn create_iir_filter(&self, feedforward: Vec<f64>, feedback: Vec<f64>) -> node::IirFilterNode {
+        let options = IirFilterOptions {
+            channel_config: ChannelConfigOptions::default(),
+            feedforward,
+            feedback,
+        };
+        node::IirFilterNode::new(self.base(), options)
+    }
+
+    /// Creates a `DelayNode`, delaying the audio signal
     fn create_delay(&self, max_delay_time: f32) -> node::DelayNode {
         let opts = node::DelayOptions {
             max_delay_time,
-            ..Default::default()
+            ..DelayOptions::default()
         };
         node::DelayNode::new(self.base(), opts)
     }
 
-    /// Creates an biquadFilterNode
+    /// Creates an `BiquadFilterNode` which implements a second order filter
     fn create_biquad_filter(&self) -> node::BiquadFilterNode {
-        node::BiquadFilterNode::new(self.base(), Default::default())
+        node::BiquadFilterNode::new(self.base(), None)
     }
 
-    /// Creates a ChannelSplitterNode
+    /// Creates a `WaveShaperNode`
+    fn create_wave_shaper(&self) -> node::WaveShaperNode {
+        node::WaveShaperNode::new(self.base(), None)
+    }
+
+    /// Creates a `ChannelSplitterNode`
     fn create_channel_splitter(&self, number_of_outputs: u32) -> node::ChannelSplitterNode {
         let opts = node::ChannelSplitterOptions {
             number_of_outputs,
-            ..Default::default()
+            ..ChannelSplitterOptions::default()
         };
         node::ChannelSplitterNode::new(self.base(), opts)
     }
 
-    /// Creates a ChannelMergerNode
+    /// Creates a `ChannelMergerNode`
     fn create_channel_merger(&self, number_of_inputs: u32) -> node::ChannelMergerNode {
         let opts = node::ChannelMergerOptions {
             number_of_inputs,
-            ..Default::default()
+            ..ChannelMergerOptions::default()
         };
         node::ChannelMergerNode::new(self.base(), opts)
     }
 
-    /// Creates a MediaStreamAudioSourceNode from a MediaElement
+    /// Creates a `MediaStreamAudioSourceNode` from a `MediaElement`
     fn create_media_stream_source<M: MediaStream>(
         &self,
         media: M,
@@ -125,7 +172,7 @@ pub trait AsBaseAudioContext {
         node::MediaStreamAudioSourceNode::new(self.base(), opts)
     }
 
-    /// Creates a MediaElementAudioSourceNode from a MediaElement
+    /// Creates a `MediaElementAudioSourceNode` from a `MediaElement`
     ///
     /// Note: do not forget to `start()` the node.
     fn create_media_element_source(
@@ -144,21 +191,21 @@ pub trait AsBaseAudioContext {
         node::MediaElementAudioSourceNode::new(self.base(), opts)
     }
 
-    /// Creates an AudioBufferSourceNode
+    /// Creates an `AudioBufferSourceNode`
     ///
     /// Note: do not forget to `start()` the node.
     fn create_buffer_source(&self) -> node::AudioBufferSourceNode {
-        node::AudioBufferSourceNode::new(self.base(), Default::default())
+        node::AudioBufferSourceNode::new(self.base(), AudioBufferSourceNodeOptions::default())
     }
 
-    /// Creates a PannerNode
+    /// Creates a `PannerNode`
     fn create_panner(&self) -> node::PannerNode {
-        node::PannerNode::new(self.base(), Default::default())
+        node::PannerNode::new(self.base(), PannerOptions::default())
     }
 
-    /// Creates a AnalyserNode
+    /// Creates a `AnalyserNode`
     fn create_analyser(&self) -> node::AnalyserNode {
-        node::AnalyserNode::new(self.base(), Default::default())
+        node::AnalyserNode::new(self.base(), AnalyserOptions::default())
     }
 
     /// Creates a periodic wave
@@ -166,9 +213,9 @@ pub trait AsBaseAudioContext {
         PeriodicWave::new(self.base(), options)
     }
 
-    /// Create an AudioParam.
+    /// Create an `AudioParam`.
     ///
-    /// Call this inside the `register` closure when setting up your AudioNode
+    /// Call this inside the `register` closure when setting up your `AudioNode`
     fn create_audio_param(
         &self,
         opts: AudioParamOptions,
@@ -187,7 +234,7 @@ pub trait AsBaseAudioContext {
         (param, proc_id)
     }
 
-    /// Returns an AudioDestinationNode representing the final destination of all audio in the
+    /// Returns an `AudioDestinationNode` representing the final destination of all audio in the
     /// context. It can be thought of as the audio-rendering device.
     fn destination(&self) -> node::DestinationNode {
         let registration = AudioContextRegistration {
@@ -200,7 +247,7 @@ pub trait AsBaseAudioContext {
         }
     }
 
-    /// Returns the AudioListener which is used for 3D spatialization
+    /// Returns the `AudioListener` which is used for 3D spatialization
     fn listener(&self) -> AudioListener {
         let mut ids = LISTENER_PARAM_IDS.map(|i| AudioContextRegistration {
             id: AudioNodeId(i),
@@ -221,7 +268,7 @@ pub trait AsBaseAudioContext {
         }
     }
 
-    /// The sample rate (in sample-frames per second) at which the AudioContext handles audio.
+    /// The sample rate (in sample-frames per second) at which the `AudioContext` handles audio.
     fn sample_rate(&self) -> SampleRate {
         self.base().sample_rate()
     }
@@ -247,9 +294,41 @@ impl AsBaseAudioContext for BaseAudioContext {
     }
 }
 
-/// This interface represents an audio graph whose AudioDestinationNode is routed to a real-time
+/// Identify the type of playback, which affects tradeoffs
+/// between audio output latency and power consumption
+pub enum LatencyHint {
+    /// Balance audio output latency and power consumption.
+    Balanced,
+    /// Provide the lowest audio output latency possible without glitching. This is the default.
+    Interactive,
+    /// Prioritize sustained playback without interruption
+    /// over audio output latency. Lowest power consumption.
+    Playback,
+    /// Specify the number of seconds of latency
+    /// this latency is not guaranted to be applied,
+    /// it depends on the audio hardware capabilities
+    Specific(f64),
+}
+
+/// Specify the playback configuration
+/// in non web context, it is the only way to specify
+/// the system configuration
+pub struct AudioContextOptions {
+    /// Identify the type of playback, which affects
+    /// tradeoffs between audio output latency and power consumption
+    pub latency_hint: Option<LatencyHint>,
+    /// Sample rate of the audio Context and audio output hardware
+    pub sample_rate: Option<u32>,
+    /// Number of output channels of destination node and audio output hardware
+    pub channels: Option<u16>,
+}
+
+/// This interface represents an audio graph whose `AudioDestinationNode` is routed to a real-time
 /// output device that produces a signal directed at the user.
+// the naming comes from the web audio specfication
+#[allow(clippy::module_name_repetitions)]
 pub struct AudioContext {
+    /// represents the underlying `BaseAudioContext`
     base: BaseAudioContext,
 
     /// cpal stream (play/pause functionality)
@@ -263,15 +342,17 @@ impl AsBaseAudioContext for AudioContext {
     }
 }
 
-/// The OfflineAudioContext doesn't render the audio to the device hardware; instead, it generates
-/// it, as fast as it can, and outputs the result to an AudioBuffer.
+/// The `OfflineAudioContext` doesn't render the audio to the device hardware; instead, it generates
+/// it, as fast as it can, and outputs the result to an `AudioBuffer`.
+// the naming comes from the web audio specfication
+#[allow(clippy::module_name_repetitions)]
 pub struct OfflineAudioContext {
+    /// represents the underlying `BaseAudioContext`
     base: BaseAudioContext,
-
     /// the size of the buffer in sample-frames
     length: usize,
     /// the rendering 'thread', fully controlled by the offline context
-    render: RenderThread,
+    renderer: RenderThread,
 }
 
 impl AsBaseAudioContext for OfflineAudioContext {
@@ -281,16 +362,19 @@ impl AsBaseAudioContext for OfflineAudioContext {
 }
 
 impl AudioContext {
-    /// Creates and returns a new AudioContext object.
+    /// Creates and returns a new `AudioContext` object.
     /// This will play live audio on the default output
+    // options is passed by value to be conform to the specification interface
+    #[allow(clippy::needless_pass_by_value)]
     #[cfg(not(test))]
-    pub fn new() -> Self {
+    #[must_use]
+    pub fn new(options: Option<AudioContextOptions>) -> Self {
         // track number of frames - synced from render thread to control thread
         let frames_played = Arc::new(AtomicU64::new(0));
         let frames_played_clone = frames_played.clone();
 
-        let (stream, config, sender) = io::build_output(frames_played_clone);
-        let channels = config.channels as u32;
+        let (stream, config, sender) = io::build_output(frames_played_clone, options.as_ref());
+        let channels = u32::from(config.channels);
         let sample_rate = SampleRate(config.sample_rate.0);
 
         let base = BaseAudioContext::new(sample_rate, channels, frames_played, sender);
@@ -299,9 +383,16 @@ impl AudioContext {
     }
 
     #[cfg(test)] // in tests, do not set up a cpal Stream
-    pub fn new() -> Self {
-        let sample_rate = SampleRate(44_100);
-        let channels = 2;
+    #[allow(clippy::must_use_candidate)]
+    pub fn new(options: Option<AudioContextOptions>) -> Self {
+        let options = options.unwrap_or(AudioContextOptions {
+            latency_hint: Some(LatencyHint::Interactive),
+            sample_rate: Some(44_100),
+            channels: Some(2),
+        });
+
+        let sample_rate = SampleRate(options.sample_rate.unwrap_or(44_100));
+        let channels = u32::from(options.channels.unwrap_or(2));
         let (sender, _receiver) = crossbeam_channel::unbounded();
         let frames_played = Arc::new(AtomicU64::new(0));
         let base = BaseAudioContext::new(sample_rate, channels, frames_played, sender);
@@ -311,33 +402,52 @@ impl AudioContext {
 
     /// Suspends the progression of time in the audio context, temporarily halting audio hardware
     /// access and reducing CPU/battery usage in the process.
+    ///
+    /// # Panics
+    ///
+    /// Will panic if:
+    ///
+    /// * The audio device is not available
+    /// * For a `BackendSpecificError`
+    // false positive due to #[cfg(not(test))]
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
     pub fn suspend(&self) {
         #[cfg(not(test))] // in tests, do not set up a cpal Stream
-        self.stream.pause().unwrap()
+        self.stream.pause().unwrap();
     }
 
     /// Resumes the progression of time in an audio context that has previously been
     /// suspended/paused.
+    ///
+    /// # Panics
+    ///
+    /// Will panic if:
+    ///
+    /// * The audio device is not available
+    /// * For a `BackendSpecificError`
+    // false positive due to #[cfg(not(test))]
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
     pub fn resume(&self) {
         #[cfg(not(test))] // in tests, do not set up a cpal Stream
-        self.stream.play().unwrap()
+        self.stream.play().expect("Audio device refuse to play");
     }
 }
 
 /// Unique identifier for audio nodes.
 ///
 /// Used for internal bookkeeping.
+#[derive(Debug)]
 pub struct AudioNodeId(u64);
 
 /// Unique identifier for audio params.
 ///
-/// Store these in your AudioProcessor to get access to AudioParam values.
+/// Store these in your `AudioProcessor` to get access to `AudioParam` values.
 pub struct AudioParamId(u64);
 
 // bit contrived, but for type safety only the context mod can access the inner u64
 impl From<&AudioParamId> for NodeIndex {
     fn from(i: &AudioParamId) -> Self {
-        NodeIndex(i.0)
+        Self(i.0)
     }
 }
 
@@ -347,14 +457,24 @@ impl From<&AudioParamId> for NodeIndex {
 ///
 /// The only way to construct this object is by calling [`BaseAudioContext::register`]
 pub struct AudioContextRegistration {
+    /// the audio context in wich nodes and connections lives
     context: BaseAudioContext,
+    /// identify a specific `AudioNode`
     id: AudioNodeId,
 }
 
 impl AudioContextRegistration {
+    /// get the audio node id of the registration
+    // false positive: AudioContextRegistration is not const
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
+    #[must_use]
     pub fn id(&self) -> &AudioNodeId {
         &self.id
     }
+    /// get the context of the registration
+    // false positive: AudioContextRegistration is not const
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
+    #[must_use]
     pub fn context(&self) -> &BaseAudioContext {
         &self.context
     }
@@ -375,6 +495,7 @@ impl Drop for AudioContextRegistration {
 }
 
 impl BaseAudioContext {
+    /// Creates a `BaseAudioContext` instance
     fn new(
         sample_rate: SampleRate,
         channels: u32,
@@ -389,7 +510,7 @@ impl BaseAudioContext {
             frames_played,
             listener_params: None,
         };
-        let base = BaseAudioContext {
+        let base = Self {
             inner: Arc::new(base_inner),
         };
 
@@ -439,25 +560,38 @@ impl BaseAudioContext {
         base
     }
 
-    /// The sample rate (in sample-frames per second) at which the AudioContext handles audio.
+    /// The sample rate (in sample-frames per second) at which the `AudioContext` handles audio.
+    #[must_use]
     pub fn sample_rate(&self) -> SampleRate {
         self.inner.sample_rate
     }
 
     /// This is the time in seconds of the sample frame immediately following the last sample-frame
     /// in the block of audio most recently processed by the context’s rendering graph.
+    #[must_use]
+    // web audio api specification requires that `current_time` returns an f64
+    // std::sync::AtomicsF64 is not currently implemented in the standard library
+    // Currently, we have no other choice than casting an u64 into f64, with possible loss of precision
+    #[allow(clippy::cast_precision_loss)]
     pub fn current_time(&self) -> f64 {
-        self.inner.frames_played.load(Ordering::SeqCst) as f64 / self.inner.sample_rate.0 as f64
+        self.inner.frames_played.load(Ordering::SeqCst) as f64 / f64::from(self.inner.sample_rate.0)
     }
 
     /// Number of channels for the audio destination
+    #[must_use]
     pub fn channels(&self) -> u32 {
         self.inner.channels
     }
 
     /// Construct a new pair of [`node::AudioNode`] and [`AudioProcessor`]
     ///
-    /// The AudioNode lives in the user-facing control thread. The Processor is sent to the render thread.
+    /// The `AudioNode` lives in the user-facing control thread. The Processor is sent to the render thread.
+    ///
+    /// # Panics
+    ///
+    /// Will panic if:
+    ///
+    /// * Message send to the render thread is not received in less than 10 ms
     pub fn register<
         T: node::AudioNode,
         F: FnOnce(AudioContextRegistration) -> (T, Box<dyn AudioProcessor>),
@@ -489,6 +623,7 @@ impl BaseAudioContext {
         node
     }
 
+    /// connects the output of the `from` audio node to the input of the `to` audio node
     pub(crate) fn connect(&self, from: &AudioNodeId, to: &AudioNodeId, output: u32, input: u32) {
         let message = ControlMessage::ConnectNode {
             from: from.0,
@@ -499,6 +634,7 @@ impl BaseAudioContext {
         self.inner.render_channel.send(message).unwrap();
     }
 
+    /// connects the `from` audio node to the `to` audio node
     pub(crate) fn disconnect(&self, from: &AudioNodeId, to: &AudioNodeId) {
         let message = ControlMessage::DisconnectNode {
             from: from.0,
@@ -507,19 +643,20 @@ impl BaseAudioContext {
         self.inner.render_channel.send(message).unwrap();
     }
 
+    /// disconnects all the audio nodes
     pub(crate) fn disconnect_all(&self, from: &AudioNodeId) {
         let message = ControlMessage::DisconnectAll { from: from.0 };
         self.inner.render_channel.send(message).unwrap();
     }
 
-    /// Pass an AudioParam AutomationEvent to the render thread
+    /// Pass an `AudioParam::AudioParamEvent` to the render thread
     ///
     /// This clunky setup (wrapping a Sender in a message sent by another Sender) ensures
     /// automation events will never be handled out of order.
     pub(crate) fn pass_audio_param_event(
         &self,
-        to: &Sender<AutomationEvent>,
-        event: AutomationEvent,
+        to: &Sender<AudioParamEvent>,
+        event: AudioParamEvent,
     ) {
         let message = ControlMessage::AudioParamEvent {
             to: to.clone(),
@@ -528,7 +665,7 @@ impl BaseAudioContext {
         self.inner.render_channel.send(message).unwrap();
     }
 
-    /// Attach the 9 AudioListener coordinates to a PannerNode
+    /// Attach the 9 `AudioListener` coordinates to a `PannerNode`
     pub(crate) fn connect_listener_to_panner(&self, panner: &AudioNodeId) {
         self.connect(&AudioNodeId(LISTENER_NODE_ID), panner, 0, 1);
         self.connect(&AudioNodeId(LISTENER_NODE_ID), panner, 1, 2);
@@ -544,11 +681,19 @@ impl BaseAudioContext {
 
 impl Default for AudioContext {
     fn default() -> Self {
-        Self::new()
+        Self::new(None)
     }
 }
 
 impl OfflineAudioContext {
+    /// Creates an `OfflineAudioContext` instance
+    ///
+    /// # Arguments
+    ///
+    /// * `channels` - number of output channels to render
+    /// * `length` - length of the rendering audio buffer
+    /// * `sample_rate` - output sample rate
+    #[must_use]
     pub fn new(channels: u32, length: usize, sample_rate: SampleRate) -> Self {
         // communication channel to the render thread
         let (sender, receiver) = crossbeam_channel::unbounded();
@@ -558,7 +703,7 @@ impl OfflineAudioContext {
         let frames_played_clone = frames_played.clone();
 
         // setup the render 'thread', which will run inside the control thread
-        let render = RenderThread::new(
+        let renderer = RenderThread::new(
             sample_rate,
             channels as usize,
             receiver,
@@ -571,20 +716,26 @@ impl OfflineAudioContext {
         Self {
             base,
             length,
-            render,
+            renderer,
         }
     }
 
+    /// `OfflineAudioContext` doesn't start rendering automatically
+    /// You need to call this function to start the audio rendering
     pub fn start_rendering(&mut self) -> AudioBuffer {
         // make buffer_size always a multiple of BUFFER_SIZE, so we can still render piecewise with
         // the desired number of frames.
-        let buffer_size = (self.length as u32 + BUFFER_SIZE - 1) / BUFFER_SIZE * BUFFER_SIZE;
+        let buffer_size = (self.length + BUFFER_SIZE - 1) / BUFFER_SIZE * BUFFER_SIZE;
 
-        let mut buf = self.render.render_audiobuffer(buffer_size as usize);
-        let _split = buf.split_off(self.length as u32);
+        let mut buf = self.renderer.render_audiobuffer(buffer_size);
+        let _split = buf.split_off(self.length);
         buf
     }
 
+    /// get the length of rendering audio buffer
+    // false positive: OfflineAudioContext is not const
+    #[allow(clippy::missing_const_for_fn, clippy::unused_self)]
+    #[must_use]
     pub fn length(&self) -> usize {
         self.length
     }
