@@ -88,11 +88,11 @@ impl RenderThread {
 
     pub fn render_audiobuffer(&mut self, length: usize) -> crate::buffer::AudioBuffer {
         // assert input was properly sized
-        debug_assert_eq!(length % BUFFER_SIZE as usize, 0);
+        debug_assert_eq!(length % BUFFER_SIZE, 0);
 
         let mut buf = crate::buffer::AudioBuffer::new(self.channels, 0, self.sample_rate);
 
-        for _ in 0..length / BUFFER_SIZE as usize {
+        for _ in 0..length / BUFFER_SIZE {
             // handle addition/removal of nodes/edges
             self.handle_control_messages();
 
@@ -118,7 +118,7 @@ impl RenderThread {
         // There may be audio frames left over from the previous render call,
         // if the cpal buffer size did not align with our internal BUFFER_SIZE
         if let Some((offset, prev_rendered)) = self.buffer_offset.take() {
-            let leftover_len = (BUFFER_SIZE as usize - offset) * self.channels;
+            let leftover_len = (BUFFER_SIZE - offset) * self.channels;
             // split the leftover frames slice, to fit in `buffer`
             let (first, next) = buffer.split_at_mut(leftover_len.min(buffer.len()));
 
@@ -144,13 +144,14 @@ impl RenderThread {
 
         // The audio graph is rendered in chunks of BUFFER_SIZE frames.  But some audio backends
         // may not be able to emit chunks of this size.
-        let chunk_size = BUFFER_SIZE as usize * self.channels as usize;
+        let chunk_size = BUFFER_SIZE * self.channels as usize;
 
         for data in buffer.chunks_mut(chunk_size) {
             // handle addition/removal of nodes/edges
             self.handle_control_messages();
 
             // update time
+            // @note - this follows the spec as fetch_add returns the old value
             let timestamp = self
                 .frames_played
                 .fetch_add(BUFFER_SIZE as u64, Ordering::SeqCst) as f64
@@ -172,7 +173,7 @@ impl RenderThread {
             if data.len() != chunk_size {
                 // this is the last chunk, and it contained less than BUFFER_SIZE samples
                 let channel_offset = data.len() / self.channels;
-                debug_assert!(channel_offset < BUFFER_SIZE as usize);
+                debug_assert!(channel_offset < BUFFER_SIZE);
                 self.buffer_offset = Some((channel_offset, rendered.clone()));
             }
         }
@@ -200,12 +201,14 @@ pub struct Node {
     has_inputs_connected: bool,
     /// Indicates if the output of this Node was consumed in the current render quantum
     has_outputs_connected: bool,
+    /// Indicates if the Node will still output a signal if no inputs are connected
+    tail_time: bool,
 }
 
 impl Node {
     /// Render an audio quantum
     fn process(&mut self, params: AudioParamValues, timestamp: f64, sample_rate: SampleRate) {
-        self.processor.process(
+        self.tail_time = self.processor.process(
             &self.inputs[..],
             &mut self.outputs[..],
             params,
@@ -229,7 +232,7 @@ impl Node {
 
         // Drop, when the node does not have any inputs connected,
         // and if the processor reports it won't yield output.
-        if !self.has_inputs_connected && !self.processor.tail_time() {
+        if !self.has_inputs_connected && !self.tail_time {
             return true;
         }
 
@@ -292,6 +295,7 @@ impl Graph {
                 free_when_finished: false,
                 has_inputs_connected: true,
                 has_outputs_connected: true,
+                tail_time: true,
             },
         );
     }
@@ -409,37 +413,49 @@ impl Graph {
         let edges = &self.edges;
         let nodes = &mut self.nodes;
 
-        // we will drop audio nodes if they are finished running
-        let mut drop_nodes = vec![];
-
+        // process every node, in topological sorted order
         ordered.iter().for_each(|index| {
-            // remove node from map, re-insert later (for borrowck reasons)
+            // remove node from graph, re-insert later (for borrowck reasons)
             let mut node = nodes.remove(index).unwrap();
+
             // for lifecycle management, check if any inputs are present
-            let mut has_inputs_connected = false;
-            // mix all inputs together
+            node.has_inputs_connected = false;
+            // and check if any outputs are present
+            // (index = 0 means this is the destination node)
+            node.has_outputs_connected = index.0 == 0;
+
+            // clear current node's inputs
             node.inputs.iter_mut().for_each(|i| i.make_silent());
 
+            // iterate all edges, find the nodes connecting to the current node,
+            // fetch their inputs and sum those together.
             edges
                 .iter()
-                .filter_map(move |(s, d)| {
-                    // audio params are connected to the 'hidden' u32::MAX input, ignore them
-                    if d.0 == *index && d.1 != u32::MAX {
-                        Some((s, d.1))
-                    } else {
-                        None
-                    }
-                })
+                .filter_map(
+                    move |(s, d)| {
+                        if d.0 == *index {
+                            Some((s, d.1))
+                        } else {
+                            None
+                        }
+                    },
+                )
                 .for_each(|(&(node_index, output), input)| {
-                    let input_node = nodes.get(&node_index).unwrap();
+                    let input_node = nodes.get_mut(&node_index).unwrap();
+                    input_node.has_outputs_connected = true;
+                    let input_node = &*input_node; // reborrow as immutable
+
+                    // audio params are connected to the 'hidden' u32::MAX input, ignore them here
+                    if input == u32::MAX {
+                        return;
+                    }
+
                     let signal = &input_node.outputs[output as usize];
-
                     node.inputs[input as usize].add(signal, node.channel_config.interpretation());
-
-                    has_inputs_connected = true;
+                    node.has_inputs_connected = true;
                 });
 
-            // up/down-mix to the desired channel count
+            // up/down-mix the cumulative inputs to the desired channel count
             let mode = node.channel_config.count_mode();
             let count = node.channel_config.count();
             let interpretation = node.channel_config.interpretation();
@@ -453,23 +469,27 @@ impl Graph {
                 input_buf.mix(new_channels, interpretation);
             });
 
+            // let the current node process
             let params = AudioParamValues::from(&*nodes);
             node.process(params, timestamp, sample_rate);
 
-            // check if the Node has reached end of lifecycle
-            node.has_inputs_connected = has_inputs_connected;
-            if node.can_free() {
-                drop_nodes.push(*index);
-            }
-
             // re-insert node in graph
             nodes.insert(*index, node);
         });
 
-        for index in drop_nodes {
-            self.remove_edges_from(index);
-            self.nodes.remove(&index);
-        }
+        // audio graph cleanup of decomissioned nodes
+        let edges = &mut self.edges;
+        let ordered = &mut self.ordered;
+        nodes.retain(|&index, node| {
+            // check if the Node has reached end of lifecycle
+            if node.can_free() {
+                edges.retain(|&(s, d)| s.0 != index && d.0 != index);
+                ordered.clear(); // void current ordering
+                false // do no retain
+            } else {
+                true
+            }
+        });
 
         // return buffer of destination node
         // assume only 1 output (todo)
@@ -492,9 +512,7 @@ mod tests {
             _params: AudioParamValues,
             _timestamp: f64,
             _sample_rate: SampleRate,
-        ) {
-        }
-        fn tail_time(&self) -> bool {
+        ) -> bool {
             false
         }
     }
