@@ -52,7 +52,7 @@ use cpal::{traits::StreamTrait, Sample, Stream};
 /// let media = sequence.map(|b| Ok(b));
 ///
 /// // media is now a proper `MediaStream` and can be used in the audio graph
-/// let context = AudioContext::new();
+/// let context = AudioContext::new(None);
 /// let node = context.create_media_stream_source(media);
 /// ```
 pub trait MediaStream:
@@ -70,7 +70,7 @@ impl<M: Iterator<Item = Result<AudioBuffer, Box<dyn Error + Send>>> + Send + 'st
 /// async executor)
 ///
 /// # Example
-/// 
+///
 /// ```rust
 /// use web_audio_api::SampleRate;
 /// use web_audio_api::context::{AudioContext, AsBaseAudioContext};
@@ -348,7 +348,7 @@ impl Iterator for Microphone {
             Err(TryRecvError::Empty) => {
                 // frame not received in time, emit silence
                 log::debug!("input frame delayed");
-                AudioBuffer::new(self.channels, BUFFER_SIZE as usize, self.sample_rate)
+                AudioBuffer::new(self.channels, BUFFER_SIZE, self.sample_rate)
             }
             Err(TryRecvError::Disconnected) => {
                 // MicrophoneRender has stopped, close stream
@@ -418,7 +418,7 @@ impl MicrophoneRender {
 /// let element = MediaElement::new(media);
 ///
 /// // register the media element node
-/// let context = AudioContext::new();
+/// let context = AudioContext::new(None);
 /// let node = context.create_media_element_source(element);
 ///
 /// // play media
@@ -474,7 +474,7 @@ impl Iterator for OggVorbisDecoder {
 /// let element = MediaElement::new(media);
 ///
 /// // register the media element node
-/// let context = AudioContext::new();
+/// let context = AudioContext::new(None);
 /// let node = context.create_media_element_source(element);
 ///
 /// // play media
@@ -527,7 +527,7 @@ impl Iterator for WavDecoder {
         for (i, res) in self
             .stream
             .by_ref()
-            .take((self.channels * crate::BUFFER_SIZE) as usize)
+            .take(self.channels as usize * BUFFER_SIZE)
             .enumerate()
         {
             match res {
