@@ -181,7 +181,7 @@ impl BiquadFilterNode {
             let default_det = 0.;
             let default_q = 1.;
 
-            let q_value = options.q.unwrap_or(default_det);
+            let q_value = options.q.unwrap_or(default_q);
             let d_value = options.detune.unwrap_or(default_det);
             let f_value = options.frequency.unwrap_or(default_freq);
             let g_value = options.gain.unwrap_or(default_gain);
@@ -497,7 +497,7 @@ impl AudioProcessor for BiquadFilterRenderer {
         params: AudioParamValues,
         _timestamp: f64,
         _sample_rate: SampleRate,
-    ) {
+    ) -> bool {
         // single input/output node
         let input = &inputs[0];
         let output = &mut outputs[0];
@@ -508,10 +508,8 @@ impl AudioProcessor for BiquadFilterRenderer {
         let q_values = params.get(&self.q);
 
         self.filter(input, output, g_values, det_values, freq_values, q_values);
-    }
 
-    fn tail_time(&self) -> bool {
-        true
+        true // todo tail time - issue #34
     }
 }
 
@@ -1108,6 +1106,7 @@ mod test {
     use float_eq::assert_float_eq;
 
     use crate::{
+        buffer::ChannelConfigOptions,
         context::{AsBaseAudioContext, OfflineAudioContext},
         node::{BiquadFilterOptions, BiquadFilterType},
         SampleRate,
@@ -1136,13 +1135,43 @@ mod test {
         let default_gain = 0.;
         let default_freq = 350.;
         let default_type = BiquadFilterType::Lowpass;
-        let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
+        let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
         let biquad = BiquadFilterNode::new(&context, None);
 
-        assert_float_eq!(biquad.q().value(), default_q, ulps <= 0);
-        assert_float_eq!(biquad.detune().value(), default_detune, ulps <= 0);
-        assert_float_eq!(biquad.gain().value(), default_gain, ulps <= 0);
-        assert_float_eq!(biquad.frequency().value(), default_freq, ulps <= 0);
+        context.start_rendering();
+
+        assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
+        assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
+        assert_float_eq!(biquad.gain().value(), default_gain, abs <= 0.);
+        assert_float_eq!(biquad.frequency().value(), default_freq, abs <= 0.);
+        assert_eq!(biquad.type_(), default_type);
+    }
+
+    #[test]
+    fn default_audio_params_are_correct_with_no_options_in_options() {
+        let default_q = 1.0;
+        let default_detune = 0.;
+        let default_gain = 0.;
+        let default_freq = 350.;
+        let default_type = BiquadFilterType::Lowpass;
+
+        let options = BiquadFilterOptions {
+            q: None,
+            detune: None,
+            frequency: None,
+            gain: None,
+            type_: None,
+            channel_config: ChannelConfigOptions::default(),
+        };
+        let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
+        let biquad = BiquadFilterNode::new(&context, Some(options));
+
+        context.start_rendering();
+
+        assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
+        assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
+        assert_float_eq!(biquad.gain().value(), default_gain, abs <= 0.);
+        assert_float_eq!(biquad.frequency().value(), default_freq, abs <= 0.);
         assert_eq!(biquad.type_(), default_type);
     }
 
@@ -1153,16 +1182,18 @@ mod test {
         let default_gain = 0.;
         let default_freq = 350.;
         let default_type = BiquadFilterType::Lowpass;
-        let context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
+        let mut context = OfflineAudioContext::new(2, LENGTH, SampleRate(44_100));
 
         let options = BiquadFilterOptions::default();
 
         let biquad = BiquadFilterNode::new(&context, Some(options));
 
-        assert_float_eq!(biquad.q().value(), default_q, ulps <= 0);
-        assert_float_eq!(biquad.detune().value(), default_detune, ulps <= 0);
-        assert_float_eq!(biquad.gain().value(), default_gain, ulps <= 0);
-        assert_float_eq!(biquad.frequency().value(), default_freq, ulps <= 0);
+        context.start_rendering();
+
+        assert_float_eq!(biquad.q().value(), default_q, abs <= 0.);
+        assert_float_eq!(biquad.detune().value(), default_detune, abs <= 0.);
+        assert_float_eq!(biquad.gain().value(), default_gain, abs <= 0.);
+        assert_float_eq!(biquad.frequency().value(), default_freq, abs <= 0.);
         assert_eq!(biquad.type_(), default_type);
     }
 
@@ -1188,10 +1219,10 @@ mod test {
 
         context.start_rendering();
 
-        assert_float_eq!(biquad.q().value(), q, ulps <= 0);
-        assert_float_eq!(biquad.detune().value(), detune, ulps <= 0);
-        assert_float_eq!(biquad.gain().value(), gain, ulps <= 0);
-        assert_float_eq!(biquad.frequency().value(), frequency, ulps <= 0);
+        assert_float_eq!(biquad.q().value(), q, abs <= 0.);
+        assert_float_eq!(biquad.detune().value(), detune, abs <= 0.);
+        assert_float_eq!(biquad.gain().value(), gain, abs <= 0.);
+        assert_float_eq!(biquad.frequency().value(), frequency, abs <= 0.);
         assert_eq!(biquad.type_(), type_);
     }
 
@@ -1214,10 +1245,10 @@ mod test {
 
         context.start_rendering();
 
-        assert_float_eq!(biquad.q().value(), q, ulps <= 0);
-        assert_float_eq!(biquad.detune().value(), detune, ulps <= 0);
-        assert_float_eq!(biquad.gain().value(), gain, ulps <= 0);
-        assert_float_eq!(biquad.frequency().value(), frequency, ulps <= 0);
+        assert_float_eq!(biquad.q().value(), q, abs <= 0.);
+        assert_float_eq!(biquad.detune().value(), detune, abs <= 0.);
+        assert_float_eq!(biquad.gain().value(), gain, abs <= 0.);
+        assert_float_eq!(biquad.frequency().value(), frequency, abs <= 0.);
         assert_eq!(biquad.type_(), type_);
     }
 
@@ -1266,6 +1297,6 @@ mod test {
         );
 
         let ref_arr = [0., niquyst];
-        assert_float_eq!(frequency_hz, ref_arr, ulps_all <= 0);
+        assert_float_eq!(frequency_hz, ref_arr, abs_all <= 0.);
     }
 }
